#include "anime.h"
#include "libmy/spr.h"


//---------------------------------------------------------------------------
ST_ANIME Anime;


//---------------------------------------------------------------------------
EWRAM_CODE void AnimeInit(void)
{
	_Memset(&Anime, 0x00, sizeof(ST_ANIME));

	AnimeReset();
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeReset(void)
{
	Anime.cnt = 0;

	SprInitOam();
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeExec(void)
{
	s32 i;

	for(i=0; i<Anime.cnt; i++)
	{
		if(Anime.reg[i].isUse == FALSE)
		{
			continue;
		}

		switch(Anime.reg[i].type)
		{
		case ANIME_TYPE_MAKE:
			AnimeExecMake(i);
			break;

		case ANIME_TYPE_MOVE:
			AnimeExecMove(i);
			break;

		case ANIME_TYPE_DISP:
			AnimeExecDisp(i);
			break;

		case ANIME_TYPE_ADD:
			AnimeExecAdd(i);
			break;
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeExecMake(u32 num)
{
	ST_ANIME_REG* p = &Anime.reg[num];


	p->var += 20;

	if(p->var >= 100)
	{
		p->var = 100;

		p->isUse = FALSE;
	}

	SpriteScaleCell(num, p->var);
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeExecMove(u32 num)
{
	ST_ANIME_REG* p = &Anime.reg[num];

	p->fsx += p->fmx;
	p->fsy += p->fmy;

	if((p->dir == KEY_UP   && p->fsy <= p->fdy) || (p->dir == KEY_DOWN  && p->fsy >= p->fdy) || 
	   (p->dir == KEY_LEFT && p->fsx <= p->fdx) || (p->dir == KEY_RIGHT && p->fsx >= p->fdx))
	{
		p->fsx = p->fdx;
		p->fsy = p->fdy;

		p->isUse = FALSE;
	}

	SpriteMoveCell(num, p->fsx, p->fsy);
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeExecDisp(u32 num)
{
	ST_ANIME_REG* p = &Anime.reg[num];

	p->isUse = FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeExecAdd(u32 num)
{
	ST_ANIME_REG* p = &Anime.reg[num];

	if(p->var < 9)
	{
		p->var++;
		return;
	}

	if(p->var < 10)
	{
		// dȂĂXvCg폜܂
		AnimeDelSprite(p->dx, p->dy);

		SpriteSetCell(num, p->chr, p->fsx , p->fsy);
		p->var++;
		return;
	}

	if(p->var >= 10 && p->var < 15)
	{
		SpriteSetCell2(num, p->chr, p->fsx - NUM2FIX(16), p->fsy - NUM2FIX(16), TRUE, p->var2);
		p->var2 += 2;
		p->var++;
		return;
	}

	if(p->var == 15)
	{
		p->var2 -= 2;
		p->var++;
		return;
	}

	if(p->var > 15 && p->var <= 20)
	{
		SpriteSetCell2(num, p->chr, p->fsx - NUM2FIX(16), p->fsy - NUM2FIX(16), TRUE, p->var2);
		p->var2 -= 2;
		p->var++;
		return;
	}

	p->isUse = FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeSetMake(s32 sx, s32 sy, u32 num)
{
	ST_ANIME_REG* p = &Anime.reg[Anime.cnt];

	p->isUse = TRUE;
	p->type  = ANIME_TYPE_MAKE;
	p->fsx   = NUM2FIX(47 + (32 + 6) * sx);
	p->fsy   = NUM2FIX( 7 + (32 + 6) * sy);
	p->chr   = SpriteGetCellChrNo(num);
	p->var   = 1;
	p->dx    = sx;
	p->dy    = sy;

	SpriteSetCell2(Anime.cnt, p->chr, p->fsx, p->fsy, FALSE, p->var);

	Anime.cnt++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeSetMove(s32 sx, s32 sy, s32 dx, s32 dy, u32 num, u16 dir)
{
	ST_ANIME_REG* p = &Anime.reg[Anime.cnt];

	p->isUse = TRUE;
	p->type  = ANIME_TYPE_MOVE;
	p->fsx   = NUM2FIX(47 + (32 + 6) * sx);
	p->fsy   = NUM2FIX( 7 + (32 + 6) * sy);
	p->fdx   = NUM2FIX(47 + (32 + 6) * dx);
	p->fdy   = NUM2FIX( 7 + (32 + 6) * dy);
	p->fmx   = NUM2FIX(10) * (dx - sx);
	p->fmy   = NUM2FIX(10) * (dy - sy);
	p->chr   = SpriteGetCellChrNo(num);
	p->dir   = dir;
	p->dx    = dx;
	p->dy    = dy;

	SpriteSetCell(Anime.cnt, p->chr, p->fsx, p->fsy);

	Anime.cnt++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeSetDisp(s32 sx, s32 sy, u32 num)
{
	ST_ANIME_REG* p = &Anime.reg[Anime.cnt];

	p->isUse = TRUE;
	p->type  = ANIME_TYPE_DISP;
	p->fsx   = NUM2FIX(47 + (32 + 6) * sx);
	p->fsy   = NUM2FIX( 7 + (32 + 6) * sy);
	p->chr   = SpriteGetCellChrNo(num);
	p->dx    = sx;
	p->dy    = sy;

	SpriteSetCell(Anime.cnt, p->chr, p->fsx, p->fsy);

	Anime.cnt++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeSetAdd(s32 sx, s32 sy, u32 num)
{
	ST_ANIME_REG* p = &Anime.reg[Anime.cnt];

	p->isUse = TRUE;
	p->type  = ANIME_TYPE_ADD;
	p->fsx   = NUM2FIX(47 + (32 + 6) * sx);
	p->fsy   = NUM2FIX( 7 + (32 + 6) * sy);
	p->chr   = SpriteGetCellChrNo(num);
	p->var   = 0;
	p->var2  = 100;
	p->dx    = sx;
	p->dy    = sy;

	Anime.cnt++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void AnimeDelSprite(s32 dx, s32 dy)
{
	s32 i;

	for(i=0; i<Anime.cnt; i++)
	{
		if(Anime.reg[i].dx == dx && Anime.reg[i].dy == dy)
		{
			SpriteMove(i, 240, 160);
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE bool AnimeIsAdd(s32 dx, s32 dy)
{
	s32 i;

	for(i=0; i<Anime.cnt; i++)
	{
		if(Anime.reg[i].type == ANIME_TYPE_ADD && Anime.reg[i].dx == dx && Anime.reg[i].dy == dy)
		{
			return TRUE;
		}
	}

	return FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool AnimeIsEnd(void)
{
	s32 i;

	for(i=0; i<Anime.cnt; i++)
	{
		if(Anime.reg[i].isUse == TRUE)
		{
			return FALSE;
		}
	}

	return TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool AnimeIsDispOnly(void)
{
	s32 i;

	for(i=0; i<Anime.cnt; i++)
	{
		if(Anime.reg[i].type != ANIME_TYPE_DISP)
		{
			return FALSE;
		}
	}

	return TRUE;
}
