#include "game.h"
#include "libmy/key.h"
#include "libmy/bg.h"
#include "panel.h"
#include "score.h"
#include "anime.h"

//---------------------------------------------------------------------------
ST_GAME Game;


//---------------------------------------------------------------------------
EWRAM_CODE void GameInit(void)
{
	_Memset(&Game, 0x00, sizeof(ST_GAME));
}
//---------------------------------------------------------------------------
IWRAM_CODE void GameExec(void)
{
	switch(Game.act)
	{
		case GAME_EXEC_RESET: GameExecReset(); break;
		case GAME_EXEC_ANIME: GameExecAnime(); break;
		case GAME_EXEC_PLAY:  GameExecPlay();  break;
		case GAME_EXEC_JUDGE: GameExecJudge(); break;
		case GAME_EXEC_OVER:  GameExecOver();  break;

		default:
			SystemError("[Game act=%x]", Game.act);
			break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void GameExecReset(void)
{
	AnimeInit();
	ScoreInit();
	PanelInit();

	PanelMakeCell();
	PanelMakeCell();
//	PanelDebug();

	GameSetAct(GAME_EXEC_ANIME);
}
//---------------------------------------------------------------------------
IWRAM_CODE void GameExecAnime(void)
{
	AnimeExec();

	if(AnimeIsEnd() == TRUE)
	{
		ScoreExec();

		if(ScoreIs2048() == TRUE || PanelIsGameOver() == TRUE)
		{
			GameSetAct(GAME_EXEC_OVER);
		}
		else
		{
			GameSetAct(GAME_EXEC_PLAY);
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void GameExecPlay(void)
{
	u16 trg = KeyGetTrg();

	if(trg & KEY_UP)
	{
		PanelMoveUp();
		GameSetAct(GAME_EXEC_JUDGE);
	}
	else if(trg & KEY_RIGHT)
	{
		PanelMoveRight();
		GameSetAct(GAME_EXEC_JUDGE);
	}
	else if(trg & KEY_DOWN)
	{
		PanelMoveDown();
		GameSetAct(GAME_EXEC_JUDGE);
	}
	else if(trg & KEY_LEFT)
	{
		PanelMoveLeft();
		GameSetAct(GAME_EXEC_JUDGE);
	}
	else if(trg & KEY_R)
	{
		ScoreSaveBest();
		GameSetAct(GAME_EXEC_RESET);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void GameExecJudge(void)
{
	if(AnimeIsDispOnly() == FALSE)
	{
		PanelMakeCell();
	}

	GameSetAct(GAME_EXEC_ANIME);
}
//---------------------------------------------------------------------------
IWRAM_CODE void GameExecOver(void)
{
	switch(Game.step)
	{
	case 0:
		BgDrawPrintf(0, 18, "GAME");

		if(ScoreIs2048() == TRUE)
		{
			BgDrawPrintf(0, 19, "CLEAR");
		}
		else
		{
			BgDrawPrintf(0, 19, "OVER");
		}
		Game.step++;
		break;

	case 1:
		if(KeyGetTrg() & KEY_A)
		{
			ScoreSaveBest();
			GameSetAct(GAME_EXEC_RESET);
		}
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void GameSetAct(u8 act)
{
	Game.act  = act;
	Game.step = 0;
}
