#include "spr.h"
#include "../res.h"

//---------------------------------------------------------------------------
u32 dummy ALIGN(4);


//---------------------------------------------------------------------------
EWRAM_CODE void SprInit(void)
{
	SprInitOam();
	SprSetDatChr();

	REG_DISPCNT |= (OBJ_ON | OBJ_2D_MAP);
}
//---------------------------------------------------------------------------
EWRAM_CODE void SprInitOam(void)
{
	REG_DMA3SAD = (u32)&dummy;
	REG_DMA3DAD = (u32)OAM;
	REG_DMA3CNT = (u32)(0x400/4) | (DMA_SRC_FIXED | DMA_DST_INC | DMA32 | DMA_ENABLE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void SprSetDat(u16* pTile, u32 tileSize, u16* pPal, u32 palSize)
{
	u32 i;

	for(i=0; i<tileSize/2; i++)
	{
		((u16*)OBJ_BASE_ADR)[i] = pTile[i];
	}

	for(i=0; i<palSize/2; i++)
	{
		((u16*)OBJ_COLORS)[i] = pPal[i];
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void SprSetDatChr(void)
{
	SprSetDat((u16*)spr_panelTiles, spr_panelTilesLen, (u16*)spr_panelPal, spr_panelPalLen);
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteMove(u32 num, s32 x, s32 y)
{
	OBJATTR* sp = (OBJATTR*)OAM + num;

	sp->attr1 &= 0xfe00;
	sp->attr0 &= 0xff00;
	sp->attr1 |= (x & 0x01ff);
	sp->attr0 |= (y & 0x00ff);
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteSetSize(u32 num, u32 size, u32 form, u32 col)
{
	OBJATTR* sp = (OBJATTR*)OAM + num;

	sp->attr0 &= 0x1fff;
	sp->attr1 &= 0x3fff;
	sp->attr0 |= col  | form | (160);
	sp->attr1 |= size | (240);
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteSetChr(u32 num, u32 chr)
{
	OBJATTR* sp = (OBJATTR*)OAM + num;

	sp->attr2 &= 0xfc00;
	sp->attr2 |= chr;
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteSetRotateFlag(u32 num, u32 no, bool isFlag)
{
	OBJATTR* sp = (OBJATTR*)OAM + num;

	sp->attr0 |= OBJ_ROT_SCALE_ON;

	sp->attr1 &= 0xC1ff;
	sp->attr1 |= OBJ_ROT_SCALE(no);
	sp->attr0 &= 0xfdff;
	sp->attr0 |= (isFlag == TRUE) ? OBJ_DOUBLE : 0;
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteSetScale(u32 num, u32 xsc, u32 ysc)
{
	OBJAFFINE* rot = (OBJAFFINE*)OAM + num;

	rot->pa = Div((256 * 100), xsc);
	rot->pb = 0;
	rot->pc = 0;
	rot->pd = Div((256 * 100), ysc);
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteSetCell(u32 num, u32 chr, s32 fx, s32 fy)
{
	SpriteSetSize(num, OBJ_SIZE(Sprite_32x32), OBJ_SQUARE, OBJ_16_COLOR);
	SpriteSetChr (num, chr);
	SpriteMove   (num, FIX2NUM(fx), FIX2NUM(fy));
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteSetCell2(u32 num, u32 chr, s32 fx, s32 fy, bool isFlag, u32 sc)
{
	SpriteSetCell(num, chr, fx, fy);

	SpriteSetRotateFlag(num, num, isFlag);
	SpriteSetScale     (num, sc, sc);
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteScaleCell(u32 num, u32 sc)
{
	SpriteSetScale(num, sc, sc);
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteMoveCell(u32 num, s32 fx, s32 fy)
{
	SpriteMove(num, FIX2NUM(fx), FIX2NUM(fy));
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpriteDelCell(u32 total, s32 fx, s32 fy)
{
	u32 i;
	u32 x = FIX2NUM(fx);
	u32 y = FIX2NUM(fy);

	for(i=0; i<total; i++)
	{
		OBJATTR* sp = (OBJATTR*)OAM + (i);

		if((sp->attr1 & 0x01ff) == x && (sp->attr0 & 0x00ff) == y)
		{
			SpriteMove(i, 240, 160);
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE u32 SpriteGetCellChrNo(u32 num)
{
	u32 ret = 0;

	switch(num)
	{
	case 2:    ret = 4;   break;
	case 4:    ret = 8;   break;
	case 8:    ret = 12;  break;
	case 16:   ret = 16;  break;
	case 32:   ret = 20;  break;
	case 64:   ret = 24;  break;
	case 128:  ret = 28;  break;
	case 256:  ret = 128; break;
	case 512:  ret = 132; break;
	case 1024: ret = 136; break;
	case 2048: ret = 140; break;

	default:
		SystemError("[AnimeGetSprChr num=%x]", num);
		break;
	}

	return ret;
}
