#include "sram.h"

//---------------------------------------------------------------------------
ST_SRAM Sram;


//---------------------------------------------------------------------------
EWRAM_CODE void SramInit(void)
{
	_Memset(&Sram, 0x00, sizeof(ST_SRAM));

	SramSeek(0);
}
//---------------------------------------------------------------------------
EWRAM_CODE void SramSeek(u32 offset)
{
	Sram.p = (u8*)SRAM + offset;
}
//---------------------------------------------------------------------------
EWRAM_CODE void SramPrev(u32 offset)
{
	Sram.p -= offset;
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 SramRead8(void)
{
	return *Sram.p++;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 SramRead16(void)
{
	u16 ret = 0;

	ret |= SramRead8();
	ret |= SramRead8() << 8;

	return ret;
}
//---------------------------------------------------------------------------
EWRAM_CODE u32 SramRead32(void)
{
	u32 ret = 0;

	ret |= SramRead8();
	ret |= SramRead8() <<  8;
	ret |= SramRead8() << 16;
	ret |= SramRead8() << 24;

	return ret;
}
//---------------------------------------------------------------------------
EWRAM_CODE void SramWrite8(u8 data)
{
	*Sram.p++ = data;
}
//---------------------------------------------------------------------------
EWRAM_CODE void SramWrite16(u16 data)
{
	SramWrite8((data      ) & 0xff);
	SramWrite8((data >>  8) & 0xff);
}
//---------------------------------------------------------------------------
EWRAM_CODE void SramWrite32(u32 data)
{
	SramWrite8((data      ) & 0xff);
	SramWrite8((data >>  8) & 0xff);
	SramWrite8((data >> 16) & 0xff);
	SramWrite8((data >> 24) & 0xff);
}
