#include "panel.h"
#include "libmy/bg.h"
#include "libmy/rnd.h"
#include "score.h"
#include "anime.h"

//---------------------------------------------------------------------------
ST_PANEL Panel;


//---------------------------------------------------------------------------
EWRAM_CODE void PanelInit(void)
{
	_Memset(&Panel, 0x00, sizeof(ST_PANEL));

	BgDrawScreen();
	BgDrawStrCls();

	BgDrawStr( 0, 0, "2048");
	BgDrawStr(25, 0, "SCORE");
	BgDrawStr(25, 3, "BEST");
}
//---------------------------------------------------------------------------
IWRAM_CODE void PanelMakeCell(void)
{
	u32 r = Rnd(PANEL_MAX_REG_CNT - Panel.regCnt);
	u32 x, y;

	for(y=0; y<PANEL_MAX_CY; y++)
	{
		for(x=0; x<PANEL_MAX_CX; x++)
		{
			if(Panel.var[x][y] == 0)
			{
				if(r == 0)
				{
					goto End;
				}
				else
				{
					r--;
				}
			}
		}
	}

End:
	_ASSERT(x < PANEL_MAX_CX && y < PANEL_MAX_CY);
	_ASSERT(Panel.var[x][y] == 0);

	Panel.var[x][y] = (Rnd(10) >= 9) ? 4 : 2;
	AnimeSetMake(x, y, Panel.var[x][y]);

	Panel.regCnt++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void PanelMoveCell(s32 sx, s32 sy, s32 x, s32 y, u16 dir)
{
	s32 dx = sx;
	s32 dy = sy;

	for(;;)
	{
		// ړ
		dx += x;
		dy += y;

		// 
		if(dx < 0 || dy < 0 || dx >= PANEL_MAX_CX || dy >= PANEL_MAX_CY)
		{
			PanelMoveCell2(sx, sy, dx - x, dy - y, dir);
			return;
		}

		// 
		if(Panel.var[dx][dy] == 0)
		{
			continue;
		}

		// Z܂́AZ
		if(Panel.var[sx][sy] != Panel.var[dx][dy] || AnimeIsAdd(dx, dy) == TRUE)
		{
			PanelMoveCell2(sx, sy, dx - x, dy - y, dir);
			return;
		}
		else
		{
			// ړւ̃Aj[Vo^܂
			AnimeSetMove(sx, sy, dx, dy, Panel.var[dx][dy], dir);

			Panel.var[sx][sy]  = 0;
			Panel.var[dx][dy] += Panel.var[dx][dy];

			// ̃Aj[Vo^܂
			AnimeSetAdd(dx, dy, Panel.var[dx][dy]);
			ScoreAddNow(Panel.var[dx][dy]);

			Panel.regCnt--;
			return;
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void PanelMoveCell2(s32 sx, s32 sy, s32 dx, s32 dy, u16 dir)
{
	// ړƈړ悪`FbN܂
	if(sx == dx && sy == dy)
	{
		// \
		AnimeSetDisp(sx, sy, Panel.var[sx][sy]);
	}
	else
	{
		// ړ
		Panel.var[dx][dy] = Panel.var[sx][sy];
		Panel.var[sx][sy] = 0;

		AnimeSetMove(sx, sy, dx, dy, Panel.var[dx][dy], dir);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void PanelMoveLeft(void)
{
	AnimeReset();

	s32 x, y;

	for(y=PANEL_MAX_CY-1; y>=0; y--)
	{
		for(x=0; x<PANEL_MAX_CX; x++)
		{
			if(Panel.var[x][y] != 0)
			{
				PanelMoveCell(x, y, -1, 0, KEY_LEFT);
			}
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void PanelMoveRight(void)
{
	AnimeReset();

	s32 x, y;

	for(y=0; y<PANEL_MAX_CY; y++)
	{
		for(x=PANEL_MAX_CX-1; x>=0; x--)
		{
			if(Panel.var[x][y] != 0)
			{
				PanelMoveCell(x, y, 1, 0, KEY_RIGHT);
			}
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void PanelMoveUp(void)
{
	AnimeReset();

	s32 x, y;

	for(x=0; x<PANEL_MAX_CX; x++)
	{
		for(y=0; y<PANEL_MAX_CY; y++)
		{
			if(Panel.var[x][y] != 0)
			{
				PanelMoveCell(x, y, 0, -1, KEY_UP);
			}
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void PanelMoveDown(void)
{
	AnimeReset();

	s32 x, y;

	for(x=PANEL_MAX_CX-1; x>=0; x--)
	{
		for(y=PANEL_MAX_CY-1; y>=0; y--)
		{
			if(Panel.var[x][y] != 0)
			{
				PanelMoveCell(x, y, 0, 1, KEY_DOWN);
			}
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE bool PanelIsGameOver(void)
{
	if(Panel.regCnt < PANEL_MAX_REG_CNT)
	{
		return FALSE;
	}

	s32 tx[4] = { 0,  1,  0, -1};
	s32 ty[4] = {-1,  0,  1,  0};
	s32 dx, dy;
	s32 x, y, i;

	for(y=0; y<PANEL_MAX_CY; y++)
	{
		for(x=0; x<PANEL_MAX_CX; x++)
		{
			for(i=0; i<4; i++)
			{
				// ړ
				dx = x + tx[i];
				dy = y + ty[i];

				if(dx < 0 || dx >= PANEL_MAX_CX)
				{
					continue;
				}

				if(dy < 0 || dy >= PANEL_MAX_CY)
				{
					continue;
				}

				if(Panel.var[x][y] == Panel.var[dx][dy])
				{
					return FALSE;
				}
			}
		}
	}

	return TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void PanelDebug(void)
{
/*
	Panel.var[0][0] = 2;
	Panel.var[0][1] = 2;
	Panel.var[0][2] = 2;
	Panel.var[0][3] = 2;

	AnimeSetMake(0, 0, 2);
	AnimeSetMake(0, 1, 2);
	AnimeSetMake(0, 2, 2);
	AnimeSetMake(0, 3, 2);

	Panel.regCnt = 4;
*/

	Panel.var[0][0] = 2;
	Panel.var[1][0] = 2;
	Panel.var[2][0] = 4;

	AnimeSetMake(0, 0, 2);
	AnimeSetMake(1, 0, 2);
	AnimeSetMake(2, 0, 4);

	Panel.regCnt = 3;
}
