#include "score.h"
#include "libmy/sram.h"
#include "libmy/rnd.h"
#include "libmy/bg.h"

//---------------------------------------------------------------------------
ST_SCORE Score;


//---------------------------------------------------------------------------
EWRAM_CODE void ScoreInit(void)
{
	_Memset(&Score, 0x00, sizeof(ST_SCORE));

	ScoreInitRnd();
	ScoreLoadBest();
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScoreInitRnd(void)
{
	SramSeek(0x10);
	u32 r = SramRead32();

	if(r != 0)
	{
		RndInitSeed(r);
	}

	SramSeek(0x10);
	SramWrite32(Rnd32());
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScoreExec(void)
{
	BgDrawPrintf(25, 1, "%5d", Score.now);
	BgDrawPrintf(25, 4, "%5d", Score.best);
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScoreSaveBest(void)
{
	if(Score.best > Score.now)
	{
		return;
	}
	Score.best = Score.now;


	SramSeek(0);

	SramWrite8('2');
	SramWrite8('0');
	SramWrite8('4');
	SramWrite8('8');

	SramWrite32(Score.best);
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScoreLoadBest(void)
{
	Score.best = 0;


	SramSeek(0);

	if(SramRead8() != '2') return;
	if(SramRead8() != '0') return;
	if(SramRead8() != '4') return;
	if(SramRead8() != '8') return;

	Score.best = SramRead32();
}
//---------------------------------------------------------------------------
IWRAM_CODE void ScoreAddNow(u32 num)
{
	if(num >= 2048)
	{
		Score.is2048 = TRUE;
	}

	Score.now += num;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool ScoreIs2048(void)
{
	return Score.is2048;
}
